-------------------------------------------------------------------------------
-- Crafting Stations
-------------------------------------------------------------------------------
CraftingStations = {}

--Libraries--------------------------------------------------------------------
local LAM = LibStub("LibAddonMenu-2.0")
local LMP = LibStub("LibMapPins-1.0")

--Local constants -------------------------------------------------------------
local ADDON_VERSION = "1.3"
local PINTYPE_NAME = "Crafting_Station_Pin"

--Local variables -------------------------------------------------------------
local savedVariables

--Default values --------------------------------------------------------------
local defaults = {
    showCraftingStations = true,
    showTamrielStations = true,
    showDivider = true,
    pinTexture = {
        type = 1,
        size = 40,
        level = 51,
    },
    pinColor = { 0.75, 1, 0.75, 1 },
    showSets = {
        [1] = true,
        [2] = true,
        [3] = true,
        [4] = true,
        [5] = true,
        [6] = true,
        [7] = true,
        [8] = true,
        [9] = true,
        [10] = true,
        [11] = true,
        [12] = true,
        [13] = true,
        [14] = true,
        [15] = true,
        [16] = true,
        [17] = true,
        [18] = true,
        [19] = true,
        [20] = true,
        [21] = true,
        [22] = true,
        [23] = true,
    }
}

-- Pin -----------------------------------------------------------------------
local pinTexturesList = {
    [1] = "Crafting Station (Default)",
    [2] = "X-Mark",
    [3] = "Small Green Dot",
    [4] = "Round Blue Glow",
    [5] = "Square Green Glow",
}

local pinTextures = {
    [1] = "esoui/art/icons/poi/poi_crafting_complete.dds",
    [2] = "CraftingStations/Icons/white_x.dds",
    [3] = "CraftingStations/Icons/green_dot.dds",
    [4] = "esoui/art/actionbar/passiveabilityframe_round_over.dds",
    [5] = "esoui/art/ava/hookpoint_oil_over.dds",
}

-- Creates Tool Tip
local pinTooltipCreator = {}
pinTooltipCreator.tooltip = 1
pinTooltipCreator.creator = function(pin)
    local _, pinTag = pin:GetPinTypeAndTag()
    local setInfo = CraftingStations.ItemSets[pinTag[3]] 

    if (savedVariables.showDivider == true) then
        ZO_Tooltip_AddDivider(InformationTooltip)
        InformationTooltip:AddLine(setInfo.name, "ZoFontGameOutline", ZO_SELECTED_TEXT:UnpackRGB())
        ZO_Tooltip_AddDivider(InformationTooltip)
        InformationTooltip:AddLine(setInfo.traits, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
        InformationTooltip:AddLine(setInfo.info, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
        ZO_Tooltip_AddDivider(InformationTooltip)
    else
        InformationTooltip:AddLine("Crafting Station", "ZoFontGameOutline", ZO_SELECTED_TEXT:UnpackRGB())
        ZO_Tooltip_AddDivider(InformationTooltip)
        InformationTooltip:AddLine(setInfo.name, "", ZO_HIGHLIGHT_TEXT:UnpackRGB())
        InformationTooltip:AddLine(setInfo.traits, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
        InformationTooltip:AddLine(setInfo.info, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
    end
end

-- Callback to Create pins
local function CreatePins()
    if not savedVariables.showCraftingStations then return end

    local zone, subzone = LMP:GetZoneAndSubzone()

    if (zone == "tamriel" and savedVariables.showTamrielStations == false) then return end

    if CraftingStations.Data[zone] and CraftingStations.Data[zone][subzone] then 
        local localStations = CraftingStations.Data[zone][subzone]

        for _, pinData in pairs(localStations) do
            local x, y, setIndex, zoneIndex, poiIndex = unpack(pinData)
            if savedVariables.showSets[setIndex] then
                if zoneIndex and poiIndex then
                    local xLoc, yLoc, _, _, isShownInCurrentMap = GetPOIMapInfo(zoneIndex, poiIndex)
                    if isShownInCurrentMap then
                        x, y = xLoc, yLoc
                    end
                end
                LMP:CreatePin(PINTYPE_NAME, pinData, x, y)
            end
        end
    end
end

-- Settings menu --------------------------------------------------------------
local function CreateSettingsMenu()
    local panelData = {
        type = "panel",
        name = "Crafting Stations",
        displayName = "|cEFEBBECrafting Stations|r",
        author = "Kottsemla (updated by Garkin)",
        version = ADDON_VERSION,
        slashCommand = "/cssettings",
        registerForRefresh = true,
        registerForDefaults = true,
    }
    local settingsPanel = LAM:RegisterAddonPanel("Crafting_Stations", panelData)

    -- Create Preview Icon
    local CreateIcons, stationPreview
    CreateIcons = function(panel)
        if panel == settingsPanel then
            stationPreview = WINDOW_MANAGER:CreateControl(nil, panel.controlsToRefresh[2], CT_TEXTURE)
            stationPreview:SetAnchor(RIGHT, panel.controlsToRefresh[2].dropdown:GetControl(), LEFT, -10, 0)
            stationPreview:SetTexture(pinTextures[savedVariables.pinTexture.type])
            stationPreview:SetDimensions(savedVariables.pinTexture.size, savedVariables.pinTexture.size)
            stationPreview:SetColor(unpack(savedVariables.pinColor))
            CALLBACK_MANAGER:UnregisterCallback("LAM-PanelControlsCreated", CreateIcons)
        end
    end
    CALLBACK_MANAGER:RegisterCallback("LAM-PanelControlsCreated", CreateIcons)

    local optionsData = {
        {
            type = "header",
            name = "GENERAL OPTIONS",
        },
        {
            type = "dropdown",
            name = "Select map pin icon",
            tooltip = "Select map pin icon",
            choices = pinTexturesList,
            getFunc = function() return pinTexturesList[savedVariables.pinTexture.type] end,
            setFunc = function(selected)
                for index, name in ipairs(pinTexturesList) do
                    if name == selected then
                        savedVariables.pinTexture.type = index
                        stationPreview:SetTexture(pinTextures[index])
                        LMP:SetLayoutKey(PINTYPE_NAME, "texture", pinTextures[index])
                        LMP:RefreshPins(PINTYPE_NAME)
                        break
                    end
                end
            end,
            default = pinTexturesList[defaults.pinTexture.type],
        },
        {
            type = "slider",
            name = "Pin size",
            tooltip = "Set the size of the map pins",
            min = 20,
            max = 70,
            getFunc = function() return savedVariables.pinTexture.size end,
            setFunc = function(size)
                savedVariables.pinTexture.size = size
                stationPreview:SetDimensions(size, size)
                LMP:SetLayoutKey(PINTYPE_NAME, "size", size)
                LMP:RefreshPins(PINTYPE_NAME)
            end,
            default = defaults.pinTexture.size,
        },
        {
            type = "colorpicker",
            name = "Pin Color",
            tooltip = "Set the color of the map pins",
            getFunc = function() return unpack(savedVariables.pinColor) end,
            setFunc = function(r,g,b,a)
                savedVariables.pinColor = {r,g,b,a}
                stationPreview:SetColor(r,g,b,a)
                LMP:GetLayoutKey(PINTYPE_NAME, "tint"):SetRGBA(r,g,b,a)
                LMP:RefreshPins(PINTYPE_NAME)
            end,
            default = ZO_ColorDef:New(unpack(defaults.pinColor))
        },
        {
            type = "header",
            name = "FILTER OPTIONS",
        },
        {
            type = "checkbox",
            name = "Show Pins On Tamriel Map",
            tooltip = "Show crafting stations on Tamriel map",
            getFunc = function() return savedVariables.showTamrielStations end,
            setFunc = function(value) savedVariables.showTamrielStations = value end,
            default = defaults.showTamrielStations,
        },
        {
            type = "header",
            name = "TOOLTIP OPTIONS",
        },
        {
            type = "checkbox",
            name = "Alternative Tooltip",
            tooltip = "Adds more dividers to tooltip. Better readability with cluttered maps",
            getFunc = function() return savedVariables.showDivider end,
            setFunc = function(value) savedVariables.showDivider = value end,
            default = function() return defaults.showDivider end,
        },
    }
    LAM:RegisterOptionControls("Crafting_Stations", optionsData)
end

-- Initialize --------------------------------------------------------------
local function OnLoad(eventCode, addOnName)

    if(addOnName == "CraftingStations") then
        
        -- Load / Create saved variable file
        savedVariables = ZO_SavedVars:New("CRAFTINGSTATIONS_DB", 1, nil, defaults)

        -- Get pin layout from saved variables
        local pinTextureType = savedVariables.pinTexture.type
        local pinTextureLevel = savedVariables.pinTexture.level
        local pinTextureSize = savedVariables.pinTexture.size
        local pinTextureTint = ZO_ColorDef:New(unpack(savedVariables.pinColor))

        local pinLayout = { level = pinTextureLevel, texture = pinTextures[pinTextureType], size = pinTextureSize, tint = pinTextureTint }
        
        -- Create addon menu
        CreateSettingsMenu()

        -- Initialize map pins (LibMapPins)
        LMP:AddPinType(PINTYPE_NAME, CreatePins, nil, pinLayout, pinTooltipCreator)

        -- Add filter check boxes
        LMP:AddPinFilter(PINTYPE_NAME, "Crafting Stations", nil, savedVariables.filters)

        -- Add handler for Left Click
        LMP:SetClickHandlers(PINTYPE_NAME, {
            [1] = {
                name = "Set waypoint to crafting station",
                show = function(pin) return true end,
                duplicates = function(pin1, pin2) return (pin1.m_PinTag[3] == pin2.m_PinTag[3]) end,
                callback = function(pin) PingMap(MAP_PIN_TYPE_PLAYER_WAYPOINT, MAP_TYPE_LOCATION_CENTERED, pin.normalizedX, pin.normalizedY) end,
            }
        })

        EVENT_MANAGER:UnregisterForEvent("CraftingStations", EVENT_ADD_ON_LOADED)
    end
end

-- Init CraftingStations
EVENT_MANAGER:RegisterForEvent("CraftingStations", EVENT_ADD_ON_LOADED, OnLoad)
